@@ -21,6 +21,7 @@
 
 require 'debian/ar'
 require 'debian/utils'
+require 'debian_version'
 
 # ruby1.6 does not have Hash.values_at, but ruby1.8 prefers it
 unless Hash.new.respond_to? :values_at
@@ -74,7 +75,7 @@ module Debian
     end
 
     def compare_versions(a, rel, b)
-      return system(DPKG, '--compare-versions', a, rel, b)
+      return Debian::Version.cmp_version(a, rel, b)
     end
 
     def field(debfile, fld=[])
@@ -135,13 +136,17 @@ module Debian
 
     def load(debfile)
       info = ''
-      Debian::Ar.new(debfile).open('control.tar.gz') {|ctz|
+      ar = Debian::Ar.new(debfile)
+      ar.open('control.tar.gz') {|ctz|
 	Debian::Utils::gunzip(ctz) {|ct|
 	  Debian::Utils::tar(ct, Debian::Utils::TAR_EXTRACT, '*/control'){|fp|
 	    info = fp.readlines.join("")
+            fp.close
 	  }
+          ct.close
 	}
       }
+      ar.close
       deb = Deb.new(info)
       deb.filename = File.expand_path(debfile, Dir.getwd)
       deb.freeze
@@ -514,8 +519,12 @@ module Debian
 	Debian::Utils.gunzip(ctz) {|ct|
 	  Debian::Utils.tar(ct, op, *pat) {|fp|
 	    if block_given?
-	      return yield(fp)
+              ct.close
+              retval = yield(fp)
+              fp.close
+	      return retval
 	    else
+              ct.close
 	      return fp
 	    end
 	  }
@@ -526,18 +535,18 @@ module Debian
     def control_fp(op, *pat)
       deb_fp("control.tar.gz", op, *pat) {|fp|
 	if block_given?
-	  return yield(fp)
+	  yield(fp)
 	else
-	  return fp
+	  fp
 	end
       }
     end
     def data_fp(op, *pat)
       deb_fp("data.tar.gz", op, *pat) {|fp|
 	if block_given?
-	  return yield(fp)
+	  yield(fp)
 	else
-	  return fp
+	  fp
 	end
       }
     end
@@ -570,14 +579,14 @@ module Debian
       end
       control_fp(Debian::Utils::TAR_EXTRACT, "*/#{cfile}") {|fp|
 	if block_given?
-	  return yield(fp)
+	  yield(fp)
 	else
-	  return fp
+	  fp
 	end
       }
     end
     def controlData(cfile = "control")
-      controlFile(cfile) {|fp| return fp.readlines.join("") }
+      controlFile(cfile) {|fp| fp.readlines.join("") }
     end
     def dataFile(fname)
       if /^\.\// =~ fname
@@ -588,14 +597,14 @@ module Debian
       end
       data_fp(Debian::Utils::TAR_EXTRACT, pat) {|fp|
 	if block_given?
-	  return yield(fp)
+	  yield(fp)
 	else
-	  return fp
+	  fp
 	end
       }
     end
     def dataData(fname)
-      dataFile(fname) {|fp| return fp.readlines.join("") }
+      dataFile(fname) {|fp| fp.readlines.join("") }
     end
     def sys_tarfile
       unless @filename || @artab
@@ -604,9 +613,9 @@ module Debian
       @artab.open("data.tar.gz") {|dtz|
 	Debian::Utils.gunzip(dtz) {|dt|
 	  if block_given?
-	    return yield(dt)
+	    yield(dt)
 	  else
-	    return dt
+	    dt
 	  end
 	}
       }
@@ -874,20 +883,14 @@ module Debian
     
     def [](package) @lists[package]; end
     def package(package) @lists[package]; end
-
-    def pkgs_re_escape(pkgs)
-      return pkgs.map{|p| Regexp.escape(p)}.join('|')
-    end
   end
 
   ################################################################
   class Sources < Archives
     def initialize(file = "", pkgs = [], fields = [])
-      unless pkgs.empty?
-	pkgre = Regexp.new("(Package|Source):\\s(#{pkgs_re_escape(pkgs)})\n")
-      end
       @lists = Archives.parseArchiveFile(file) {|info|
-	if !pkgre || pkgre =~ info
+        info =~ /(?:Package|Source):\s(.*)$/;
+	if pkgs.empty? || pkgs.include?($1)
 	  d = Dsc.new(info,fields)
 	  if block_given?
 	    yield d
@@ -901,13 +904,11 @@ module Debian
   ################################################################  
   class Packages < Archives
     def initialize(file = "", pkgs = [], fields = [])
-      unless pkgs.empty?
-	pkgre = Regexp.new("Package:\\s(#{pkgs_re_escape(pkgs)})\n")
-      end
       @provides = {}
       @file = [file]
       @lists = Archives.parseArchiveFile(file) {|info|
-	if !pkgre || pkgre =~ info
+        info =~ /Package:\s(.*)$/;
+	if pkgs.empty? || pkgs.include?($1)
 	  d = Deb.new(info,fields)
 	  add_provides(d)
 	  if block_given?
